#!perl

use 5.008001;
use strict;
## We cannot 'use warnings' here as PG_TSQUERY and others trip it up
## no critic (RequireUseWarnings)
use lib 'blib/lib', 'blib/arch', 't';
use Test::More;
plan tests => 1;
select(($|=1,select(STDERR),$|=1)[1]);

use DBD::Pg qw(:pg_types :async);

## Should match the list in Pg.xs
## This is auto-generated by types.c, so do not edit manually please

my $fails=0;

## no critic (Lax::RequireConstantOnLeftSideOfEquality::ExceptEq)

PG_ACLITEM                        ==  1033 or (++$fails && fail ('PG_ACLITEM returned wrong value: expected 1033, got ' . PG_ACLITEM));
PG_ACLITEMARRAY                   ==  1034 or (++$fails && fail ('PG_ACLITEMARRAY returned wrong value: expected 1034, got ' . PG_ACLITEMARRAY));
PG_ANY                            ==  2276 or (++$fails && fail ('PG_ANY returned wrong value: expected 2276, got ' . PG_ANY));
PG_ANYARRAY                       ==  2277 or (++$fails && fail ('PG_ANYARRAY returned wrong value: expected 2277, got ' . PG_ANYARRAY));
PG_ANYCOMPATIBLE                  ==  5077 or (++$fails && fail ('PG_ANYCOMPATIBLE returned wrong value: expected 5077, got ' . PG_ANYCOMPATIBLE));
PG_ANYCOMPATIBLEARRAY             ==  5078 or (++$fails && fail ('PG_ANYCOMPATIBLEARRAY returned wrong value: expected 5078, got ' . PG_ANYCOMPATIBLEARRAY));
PG_ANYCOMPATIBLEMULTIRANGE        ==  4538 or (++$fails && fail ('PG_ANYCOMPATIBLEMULTIRANGE returned wrong value: expected 4538, got ' . PG_ANYCOMPATIBLEMULTIRANGE));
PG_ANYCOMPATIBLENONARRAY          ==  5079 or (++$fails && fail ('PG_ANYCOMPATIBLENONARRAY returned wrong value: expected 5079, got ' . PG_ANYCOMPATIBLENONARRAY));
PG_ANYCOMPATIBLERANGE             ==  5080 or (++$fails && fail ('PG_ANYCOMPATIBLERANGE returned wrong value: expected 5080, got ' . PG_ANYCOMPATIBLERANGE));
PG_ANYELEMENT                     ==  2283 or (++$fails && fail ('PG_ANYELEMENT returned wrong value: expected 2283, got ' . PG_ANYELEMENT));
PG_ANYENUM                        ==  3500 or (++$fails && fail ('PG_ANYENUM returned wrong value: expected 3500, got ' . PG_ANYENUM));
PG_ANYMULTIRANGE                  ==  4537 or (++$fails && fail ('PG_ANYMULTIRANGE returned wrong value: expected 4537, got ' . PG_ANYMULTIRANGE));
PG_ANYNONARRAY                    ==  2776 or (++$fails && fail ('PG_ANYNONARRAY returned wrong value: expected 2776, got ' . PG_ANYNONARRAY));
PG_ANYRANGE                       ==  3831 or (++$fails && fail ('PG_ANYRANGE returned wrong value: expected 3831, got ' . PG_ANYRANGE));
PG_BIT                            ==  1560 or (++$fails && fail ('PG_BIT returned wrong value: expected 1560, got ' . PG_BIT));
PG_BITARRAY                       ==  1561 or (++$fails && fail ('PG_BITARRAY returned wrong value: expected 1561, got ' . PG_BITARRAY));
PG_BOOL                           ==    16 or (++$fails && fail ('PG_BOOL returned wrong value: expected 16, got ' . PG_BOOL));
PG_BOOLARRAY                      ==  1000 or (++$fails && fail ('PG_BOOLARRAY returned wrong value: expected 1000, got ' . PG_BOOLARRAY));
PG_BOX                            ==   603 or (++$fails && fail ('PG_BOX returned wrong value: expected 603, got ' . PG_BOX));
PG_BOXARRAY                       ==  1020 or (++$fails && fail ('PG_BOXARRAY returned wrong value: expected 1020, got ' . PG_BOXARRAY));
PG_BPCHAR                         ==  1042 or (++$fails && fail ('PG_BPCHAR returned wrong value: expected 1042, got ' . PG_BPCHAR));
PG_BPCHARARRAY                    ==  1014 or (++$fails && fail ('PG_BPCHARARRAY returned wrong value: expected 1014, got ' . PG_BPCHARARRAY));
PG_BYTEA                          ==    17 or (++$fails && fail ('PG_BYTEA returned wrong value: expected 17, got ' . PG_BYTEA));
PG_BYTEAARRAY                     ==  1001 or (++$fails && fail ('PG_BYTEAARRAY returned wrong value: expected 1001, got ' . PG_BYTEAARRAY));
PG_CHAR                           ==    18 or (++$fails && fail ('PG_CHAR returned wrong value: expected 18, got ' . PG_CHAR));
PG_CHARARRAY                      ==  1002 or (++$fails && fail ('PG_CHARARRAY returned wrong value: expected 1002, got ' . PG_CHARARRAY));
PG_CID                            ==    29 or (++$fails && fail ('PG_CID returned wrong value: expected 29, got ' . PG_CID));
PG_CIDARRAY                       ==  1012 or (++$fails && fail ('PG_CIDARRAY returned wrong value: expected 1012, got ' . PG_CIDARRAY));
PG_CIDR                           ==   650 or (++$fails && fail ('PG_CIDR returned wrong value: expected 650, got ' . PG_CIDR));
PG_CIDRARRAY                      ==   651 or (++$fails && fail ('PG_CIDRARRAY returned wrong value: expected 651, got ' . PG_CIDRARRAY));
PG_CIRCLE                         ==   718 or (++$fails && fail ('PG_CIRCLE returned wrong value: expected 718, got ' . PG_CIRCLE));
PG_CIRCLEARRAY                    ==   719 or (++$fails && fail ('PG_CIRCLEARRAY returned wrong value: expected 719, got ' . PG_CIRCLEARRAY));
PG_CSTRING                        ==  2275 or (++$fails && fail ('PG_CSTRING returned wrong value: expected 2275, got ' . PG_CSTRING));
PG_CSTRINGARRAY                   ==  1263 or (++$fails && fail ('PG_CSTRINGARRAY returned wrong value: expected 1263, got ' . PG_CSTRINGARRAY));
PG_DATE                           ==  1082 or (++$fails && fail ('PG_DATE returned wrong value: expected 1082, got ' . PG_DATE));
PG_DATEARRAY                      ==  1182 or (++$fails && fail ('PG_DATEARRAY returned wrong value: expected 1182, got ' . PG_DATEARRAY));
PG_DATEMULTIRANGE                 ==  4535 or (++$fails && fail ('PG_DATEMULTIRANGE returned wrong value: expected 4535, got ' . PG_DATEMULTIRANGE));
PG_DATEMULTIRANGEARRAY            ==  6155 or (++$fails && fail ('PG_DATEMULTIRANGEARRAY returned wrong value: expected 6155, got ' . PG_DATEMULTIRANGEARRAY));
PG_DATERANGE                      ==  3912 or (++$fails && fail ('PG_DATERANGE returned wrong value: expected 3912, got ' . PG_DATERANGE));
PG_DATERANGEARRAY                 ==  3913 or (++$fails && fail ('PG_DATERANGEARRAY returned wrong value: expected 3913, got ' . PG_DATERANGEARRAY));
PG_EVENT_TRIGGER                  ==  3838 or (++$fails && fail ('PG_EVENT_TRIGGER returned wrong value: expected 3838, got ' . PG_EVENT_TRIGGER));
PG_FDW_HANDLER                    ==  3115 or (++$fails && fail ('PG_FDW_HANDLER returned wrong value: expected 3115, got ' . PG_FDW_HANDLER));
PG_FLOAT4                         ==   700 or (++$fails && fail ('PG_FLOAT4 returned wrong value: expected 700, got ' . PG_FLOAT4));
PG_FLOAT4ARRAY                    ==  1021 or (++$fails && fail ('PG_FLOAT4ARRAY returned wrong value: expected 1021, got ' . PG_FLOAT4ARRAY));
PG_FLOAT8                         ==   701 or (++$fails && fail ('PG_FLOAT8 returned wrong value: expected 701, got ' . PG_FLOAT8));
PG_FLOAT8ARRAY                    ==  1022 or (++$fails && fail ('PG_FLOAT8ARRAY returned wrong value: expected 1022, got ' . PG_FLOAT8ARRAY));
PG_GTSVECTOR                      ==  3642 or (++$fails && fail ('PG_GTSVECTOR returned wrong value: expected 3642, got ' . PG_GTSVECTOR));
PG_GTSVECTORARRAY                 ==  3644 or (++$fails && fail ('PG_GTSVECTORARRAY returned wrong value: expected 3644, got ' . PG_GTSVECTORARRAY));
PG_INDEX_AM_HANDLER               ==   325 or (++$fails && fail ('PG_INDEX_AM_HANDLER returned wrong value: expected 325, got ' . PG_INDEX_AM_HANDLER));
PG_INET                           ==   869 or (++$fails && fail ('PG_INET returned wrong value: expected 869, got ' . PG_INET));
PG_INETARRAY                      ==  1041 or (++$fails && fail ('PG_INETARRAY returned wrong value: expected 1041, got ' . PG_INETARRAY));
PG_INT2                           ==    21 or (++$fails && fail ('PG_INT2 returned wrong value: expected 21, got ' . PG_INT2));
PG_INT2ARRAY                      ==  1005 or (++$fails && fail ('PG_INT2ARRAY returned wrong value: expected 1005, got ' . PG_INT2ARRAY));
PG_INT2VECTOR                     ==    22 or (++$fails && fail ('PG_INT2VECTOR returned wrong value: expected 22, got ' . PG_INT2VECTOR));
PG_INT2VECTORARRAY                ==  1006 or (++$fails && fail ('PG_INT2VECTORARRAY returned wrong value: expected 1006, got ' . PG_INT2VECTORARRAY));
PG_INT4                           ==    23 or (++$fails && fail ('PG_INT4 returned wrong value: expected 23, got ' . PG_INT4));
PG_INT4ARRAY                      ==  1007 or (++$fails && fail ('PG_INT4ARRAY returned wrong value: expected 1007, got ' . PG_INT4ARRAY));
PG_INT4MULTIRANGE                 ==  4451 or (++$fails && fail ('PG_INT4MULTIRANGE returned wrong value: expected 4451, got ' . PG_INT4MULTIRANGE));
PG_INT4MULTIRANGEARRAY            ==  6150 or (++$fails && fail ('PG_INT4MULTIRANGEARRAY returned wrong value: expected 6150, got ' . PG_INT4MULTIRANGEARRAY));
PG_INT4RANGE                      ==  3904 or (++$fails && fail ('PG_INT4RANGE returned wrong value: expected 3904, got ' . PG_INT4RANGE));
PG_INT4RANGEARRAY                 ==  3905 or (++$fails && fail ('PG_INT4RANGEARRAY returned wrong value: expected 3905, got ' . PG_INT4RANGEARRAY));
PG_INT8                           ==    20 or (++$fails && fail ('PG_INT8 returned wrong value: expected 20, got ' . PG_INT8));
PG_INT8ARRAY                      ==  1016 or (++$fails && fail ('PG_INT8ARRAY returned wrong value: expected 1016, got ' . PG_INT8ARRAY));
PG_INT8MULTIRANGE                 ==  4536 or (++$fails && fail ('PG_INT8MULTIRANGE returned wrong value: expected 4536, got ' . PG_INT8MULTIRANGE));
PG_INT8MULTIRANGEARRAY            ==  6157 or (++$fails && fail ('PG_INT8MULTIRANGEARRAY returned wrong value: expected 6157, got ' . PG_INT8MULTIRANGEARRAY));
PG_INT8RANGE                      ==  3926 or (++$fails && fail ('PG_INT8RANGE returned wrong value: expected 3926, got ' . PG_INT8RANGE));
PG_INT8RANGEARRAY                 ==  3927 or (++$fails && fail ('PG_INT8RANGEARRAY returned wrong value: expected 3927, got ' . PG_INT8RANGEARRAY));
PG_INTERNAL                       ==  2281 or (++$fails && fail ('PG_INTERNAL returned wrong value: expected 2281, got ' . PG_INTERNAL));
PG_INTERVAL                       ==  1186 or (++$fails && fail ('PG_INTERVAL returned wrong value: expected 1186, got ' . PG_INTERVAL));
PG_INTERVALARRAY                  ==  1187 or (++$fails && fail ('PG_INTERVALARRAY returned wrong value: expected 1187, got ' . PG_INTERVALARRAY));
PG_JSON                           ==   114 or (++$fails && fail ('PG_JSON returned wrong value: expected 114, got ' . PG_JSON));
PG_JSONARRAY                      ==   199 or (++$fails && fail ('PG_JSONARRAY returned wrong value: expected 199, got ' . PG_JSONARRAY));
PG_JSONB                          ==  3802 or (++$fails && fail ('PG_JSONB returned wrong value: expected 3802, got ' . PG_JSONB));
PG_JSONBARRAY                     ==  3807 or (++$fails && fail ('PG_JSONBARRAY returned wrong value: expected 3807, got ' . PG_JSONBARRAY));
PG_JSONPATH                       ==  4072 or (++$fails && fail ('PG_JSONPATH returned wrong value: expected 4072, got ' . PG_JSONPATH));
PG_JSONPATHARRAY                  ==  4073 or (++$fails && fail ('PG_JSONPATHARRAY returned wrong value: expected 4073, got ' . PG_JSONPATHARRAY));
PG_LANGUAGE_HANDLER               ==  2280 or (++$fails && fail ('PG_LANGUAGE_HANDLER returned wrong value: expected 2280, got ' . PG_LANGUAGE_HANDLER));
PG_LINE                           ==   628 or (++$fails && fail ('PG_LINE returned wrong value: expected 628, got ' . PG_LINE));
PG_LINEARRAY                      ==   629 or (++$fails && fail ('PG_LINEARRAY returned wrong value: expected 629, got ' . PG_LINEARRAY));
PG_LSEG                           ==   601 or (++$fails && fail ('PG_LSEG returned wrong value: expected 601, got ' . PG_LSEG));
PG_LSEGARRAY                      ==  1018 or (++$fails && fail ('PG_LSEGARRAY returned wrong value: expected 1018, got ' . PG_LSEGARRAY));
PG_MACADDR                        ==   829 or (++$fails && fail ('PG_MACADDR returned wrong value: expected 829, got ' . PG_MACADDR));
PG_MACADDR8                       ==   774 or (++$fails && fail ('PG_MACADDR8 returned wrong value: expected 774, got ' . PG_MACADDR8));
PG_MACADDR8ARRAY                  ==   775 or (++$fails && fail ('PG_MACADDR8ARRAY returned wrong value: expected 775, got ' . PG_MACADDR8ARRAY));
PG_MACADDRARRAY                   ==  1040 or (++$fails && fail ('PG_MACADDRARRAY returned wrong value: expected 1040, got ' . PG_MACADDRARRAY));
PG_MONEY                          ==   790 or (++$fails && fail ('PG_MONEY returned wrong value: expected 790, got ' . PG_MONEY));
PG_MONEYARRAY                     ==   791 or (++$fails && fail ('PG_MONEYARRAY returned wrong value: expected 791, got ' . PG_MONEYARRAY));
PG_NAME                           ==    19 or (++$fails && fail ('PG_NAME returned wrong value: expected 19, got ' . PG_NAME));
PG_NAMEARRAY                      ==  1003 or (++$fails && fail ('PG_NAMEARRAY returned wrong value: expected 1003, got ' . PG_NAMEARRAY));
PG_NUMERIC                        ==  1700 or (++$fails && fail ('PG_NUMERIC returned wrong value: expected 1700, got ' . PG_NUMERIC));
PG_NUMERICARRAY                   ==  1231 or (++$fails && fail ('PG_NUMERICARRAY returned wrong value: expected 1231, got ' . PG_NUMERICARRAY));
PG_NUMMULTIRANGE                  ==  4532 or (++$fails && fail ('PG_NUMMULTIRANGE returned wrong value: expected 4532, got ' . PG_NUMMULTIRANGE));
PG_NUMMULTIRANGEARRAY             ==  6151 or (++$fails && fail ('PG_NUMMULTIRANGEARRAY returned wrong value: expected 6151, got ' . PG_NUMMULTIRANGEARRAY));
PG_NUMRANGE                       ==  3906 or (++$fails && fail ('PG_NUMRANGE returned wrong value: expected 3906, got ' . PG_NUMRANGE));
PG_NUMRANGEARRAY                  ==  3907 or (++$fails && fail ('PG_NUMRANGEARRAY returned wrong value: expected 3907, got ' . PG_NUMRANGEARRAY));
PG_OID                            ==    26 or (++$fails && fail ('PG_OID returned wrong value: expected 26, got ' . PG_OID));
PG_OID8                           ==  6437 or (++$fails && fail ('PG_OID8 returned wrong value: expected 6437, got ' . PG_OID8));
PG_OID8ARRAY                      ==  6442 or (++$fails && fail ('PG_OID8ARRAY returned wrong value: expected 6442, got ' . PG_OID8ARRAY));
PG_OIDARRAY                       ==  1028 or (++$fails && fail ('PG_OIDARRAY returned wrong value: expected 1028, got ' . PG_OIDARRAY));
PG_OIDVECTOR                      ==    30 or (++$fails && fail ('PG_OIDVECTOR returned wrong value: expected 30, got ' . PG_OIDVECTOR));
PG_OIDVECTORARRAY                 ==  1013 or (++$fails && fail ('PG_OIDVECTORARRAY returned wrong value: expected 1013, got ' . PG_OIDVECTORARRAY));
PG_PATH                           ==   602 or (++$fails && fail ('PG_PATH returned wrong value: expected 602, got ' . PG_PATH));
PG_PATHARRAY                      ==  1019 or (++$fails && fail ('PG_PATHARRAY returned wrong value: expected 1019, got ' . PG_PATHARRAY));
PG_PG_ATTRIBUTE                   ==    75 or (++$fails && fail ('PG_PG_ATTRIBUTE returned wrong value: expected 75, got ' . PG_PG_ATTRIBUTE));
PG_PG_ATTRIBUTEARRAY              ==   270 or (++$fails && fail ('PG_PG_ATTRIBUTEARRAY returned wrong value: expected 270, got ' . PG_PG_ATTRIBUTEARRAY));
PG_PG_BRIN_BLOOM_SUMMARY          ==  4600 or (++$fails && fail ('PG_PG_BRIN_BLOOM_SUMMARY returned wrong value: expected 4600, got ' . PG_PG_BRIN_BLOOM_SUMMARY));
PG_PG_BRIN_MINMAX_MULTI_SUMMARY   ==  4601 or (++$fails && fail ('PG_PG_BRIN_MINMAX_MULTI_SUMMARY returned wrong value: expected 4601, got ' . PG_PG_BRIN_MINMAX_MULTI_SUMMARY));
PG_PG_CLASS                       ==    83 or (++$fails && fail ('PG_PG_CLASS returned wrong value: expected 83, got ' . PG_PG_CLASS));
PG_PG_CLASSARRAY                  ==   273 or (++$fails && fail ('PG_PG_CLASSARRAY returned wrong value: expected 273, got ' . PG_PG_CLASSARRAY));
PG_PG_DDL_COMMAND                 ==    32 or (++$fails && fail ('PG_PG_DDL_COMMAND returned wrong value: expected 32, got ' . PG_PG_DDL_COMMAND));
PG_PG_DEPENDENCIES                ==  3402 or (++$fails && fail ('PG_PG_DEPENDENCIES returned wrong value: expected 3402, got ' . PG_PG_DEPENDENCIES));
PG_PG_LSN                         ==  3220 or (++$fails && fail ('PG_PG_LSN returned wrong value: expected 3220, got ' . PG_PG_LSN));
PG_PG_LSNARRAY                    ==  3221 or (++$fails && fail ('PG_PG_LSNARRAY returned wrong value: expected 3221, got ' . PG_PG_LSNARRAY));
PG_PG_MCV_LIST                    ==  5017 or (++$fails && fail ('PG_PG_MCV_LIST returned wrong value: expected 5017, got ' . PG_PG_MCV_LIST));
PG_PG_NDISTINCT                   ==  3361 or (++$fails && fail ('PG_PG_NDISTINCT returned wrong value: expected 3361, got ' . PG_PG_NDISTINCT));
PG_PG_NODE_TREE                   ==   194 or (++$fails && fail ('PG_PG_NODE_TREE returned wrong value: expected 194, got ' . PG_PG_NODE_TREE));
PG_PG_PROC                        ==    81 or (++$fails && fail ('PG_PG_PROC returned wrong value: expected 81, got ' . PG_PG_PROC));
PG_PG_PROCARRAY                   ==   272 or (++$fails && fail ('PG_PG_PROCARRAY returned wrong value: expected 272, got ' . PG_PG_PROCARRAY));
PG_PG_SNAPSHOT                    ==  5038 or (++$fails && fail ('PG_PG_SNAPSHOT returned wrong value: expected 5038, got ' . PG_PG_SNAPSHOT));
PG_PG_SNAPSHOTARRAY               ==  5039 or (++$fails && fail ('PG_PG_SNAPSHOTARRAY returned wrong value: expected 5039, got ' . PG_PG_SNAPSHOTARRAY));
PG_PG_TYPE                        ==    71 or (++$fails && fail ('PG_PG_TYPE returned wrong value: expected 71, got ' . PG_PG_TYPE));
PG_PG_TYPEARRAY                   ==   210 or (++$fails && fail ('PG_PG_TYPEARRAY returned wrong value: expected 210, got ' . PG_PG_TYPEARRAY));
PG_POINT                          ==   600 or (++$fails && fail ('PG_POINT returned wrong value: expected 600, got ' . PG_POINT));
PG_POINTARRAY                     ==  1017 or (++$fails && fail ('PG_POINTARRAY returned wrong value: expected 1017, got ' . PG_POINTARRAY));
PG_POLYGON                        ==   604 or (++$fails && fail ('PG_POLYGON returned wrong value: expected 604, got ' . PG_POLYGON));
PG_POLYGONARRAY                   ==  1027 or (++$fails && fail ('PG_POLYGONARRAY returned wrong value: expected 1027, got ' . PG_POLYGONARRAY));
PG_RECORD                         ==  2249 or (++$fails && fail ('PG_RECORD returned wrong value: expected 2249, got ' . PG_RECORD));
PG_RECORDARRAY                    ==  2287 or (++$fails && fail ('PG_RECORDARRAY returned wrong value: expected 2287, got ' . PG_RECORDARRAY));
PG_REFCURSOR                      ==  1790 or (++$fails && fail ('PG_REFCURSOR returned wrong value: expected 1790, got ' . PG_REFCURSOR));
PG_REFCURSORARRAY                 ==  2201 or (++$fails && fail ('PG_REFCURSORARRAY returned wrong value: expected 2201, got ' . PG_REFCURSORARRAY));
PG_REGCLASS                       ==  2205 or (++$fails && fail ('PG_REGCLASS returned wrong value: expected 2205, got ' . PG_REGCLASS));
PG_REGCLASSARRAY                  ==  2210 or (++$fails && fail ('PG_REGCLASSARRAY returned wrong value: expected 2210, got ' . PG_REGCLASSARRAY));
PG_REGCOLLATION                   ==  4191 or (++$fails && fail ('PG_REGCOLLATION returned wrong value: expected 4191, got ' . PG_REGCOLLATION));
PG_REGCOLLATIONARRAY              ==  4192 or (++$fails && fail ('PG_REGCOLLATIONARRAY returned wrong value: expected 4192, got ' . PG_REGCOLLATIONARRAY));
PG_REGCONFIG                      ==  3734 or (++$fails && fail ('PG_REGCONFIG returned wrong value: expected 3734, got ' . PG_REGCONFIG));
PG_REGCONFIGARRAY                 ==  3735 or (++$fails && fail ('PG_REGCONFIGARRAY returned wrong value: expected 3735, got ' . PG_REGCONFIGARRAY));
PG_REGDATABASE                    ==  6490 or (++$fails && fail ('PG_REGDATABASE returned wrong value: expected 6490, got ' . PG_REGDATABASE));
PG_REGDATABASEARRAY               ==  6491 or (++$fails && fail ('PG_REGDATABASEARRAY returned wrong value: expected 6491, got ' . PG_REGDATABASEARRAY));
PG_REGDICTIONARY                  ==  3769 or (++$fails && fail ('PG_REGDICTIONARY returned wrong value: expected 3769, got ' . PG_REGDICTIONARY));
PG_REGDICTIONARYARRAY             ==  3770 or (++$fails && fail ('PG_REGDICTIONARYARRAY returned wrong value: expected 3770, got ' . PG_REGDICTIONARYARRAY));
PG_REGNAMESPACE                   ==  4089 or (++$fails && fail ('PG_REGNAMESPACE returned wrong value: expected 4089, got ' . PG_REGNAMESPACE));
PG_REGNAMESPACEARRAY              ==  4090 or (++$fails && fail ('PG_REGNAMESPACEARRAY returned wrong value: expected 4090, got ' . PG_REGNAMESPACEARRAY));
PG_REGOPER                        ==  2203 or (++$fails && fail ('PG_REGOPER returned wrong value: expected 2203, got ' . PG_REGOPER));
PG_REGOPERARRAY                   ==  2208 or (++$fails && fail ('PG_REGOPERARRAY returned wrong value: expected 2208, got ' . PG_REGOPERARRAY));
PG_REGOPERATOR                    ==  2204 or (++$fails && fail ('PG_REGOPERATOR returned wrong value: expected 2204, got ' . PG_REGOPERATOR));
PG_REGOPERATORARRAY               ==  2209 or (++$fails && fail ('PG_REGOPERATORARRAY returned wrong value: expected 2209, got ' . PG_REGOPERATORARRAY));
PG_REGPROC                        ==    24 or (++$fails && fail ('PG_REGPROC returned wrong value: expected 24, got ' . PG_REGPROC));
PG_REGPROCARRAY                   ==  1008 or (++$fails && fail ('PG_REGPROCARRAY returned wrong value: expected 1008, got ' . PG_REGPROCARRAY));
PG_REGPROCEDURE                   ==  2202 or (++$fails && fail ('PG_REGPROCEDURE returned wrong value: expected 2202, got ' . PG_REGPROCEDURE));
PG_REGPROCEDUREARRAY              ==  2207 or (++$fails && fail ('PG_REGPROCEDUREARRAY returned wrong value: expected 2207, got ' . PG_REGPROCEDUREARRAY));
PG_REGROLE                        ==  4096 or (++$fails && fail ('PG_REGROLE returned wrong value: expected 4096, got ' . PG_REGROLE));
PG_REGROLEARRAY                   ==  4097 or (++$fails && fail ('PG_REGROLEARRAY returned wrong value: expected 4097, got ' . PG_REGROLEARRAY));
PG_REGTYPE                        ==  2206 or (++$fails && fail ('PG_REGTYPE returned wrong value: expected 2206, got ' . PG_REGTYPE));
PG_REGTYPEARRAY                   ==  2211 or (++$fails && fail ('PG_REGTYPEARRAY returned wrong value: expected 2211, got ' . PG_REGTYPEARRAY));
PG_TABLE_AM_HANDLER               ==   269 or (++$fails && fail ('PG_TABLE_AM_HANDLER returned wrong value: expected 269, got ' . PG_TABLE_AM_HANDLER));
PG_TEXT                           ==    25 or (++$fails && fail ('PG_TEXT returned wrong value: expected 25, got ' . PG_TEXT));
PG_TEXTARRAY                      ==  1009 or (++$fails && fail ('PG_TEXTARRAY returned wrong value: expected 1009, got ' . PG_TEXTARRAY));
PG_TID                            ==    27 or (++$fails && fail ('PG_TID returned wrong value: expected 27, got ' . PG_TID));
PG_TIDARRAY                       ==  1010 or (++$fails && fail ('PG_TIDARRAY returned wrong value: expected 1010, got ' . PG_TIDARRAY));
PG_TIME                           ==  1083 or (++$fails && fail ('PG_TIME returned wrong value: expected 1083, got ' . PG_TIME));
PG_TIMEARRAY                      ==  1183 or (++$fails && fail ('PG_TIMEARRAY returned wrong value: expected 1183, got ' . PG_TIMEARRAY));
PG_TIMESTAMP                      ==  1114 or (++$fails && fail ('PG_TIMESTAMP returned wrong value: expected 1114, got ' . PG_TIMESTAMP));
PG_TIMESTAMPARRAY                 ==  1115 or (++$fails && fail ('PG_TIMESTAMPARRAY returned wrong value: expected 1115, got ' . PG_TIMESTAMPARRAY));
PG_TIMESTAMPTZ                    ==  1184 or (++$fails && fail ('PG_TIMESTAMPTZ returned wrong value: expected 1184, got ' . PG_TIMESTAMPTZ));
PG_TIMESTAMPTZARRAY               ==  1185 or (++$fails && fail ('PG_TIMESTAMPTZARRAY returned wrong value: expected 1185, got ' . PG_TIMESTAMPTZARRAY));
PG_TIMETZ                         ==  1266 or (++$fails && fail ('PG_TIMETZ returned wrong value: expected 1266, got ' . PG_TIMETZ));
PG_TIMETZARRAY                    ==  1270 or (++$fails && fail ('PG_TIMETZARRAY returned wrong value: expected 1270, got ' . PG_TIMETZARRAY));
PG_TRIGGER                        ==  2279 or (++$fails && fail ('PG_TRIGGER returned wrong value: expected 2279, got ' . PG_TRIGGER));
PG_TSMULTIRANGE                   ==  4533 or (++$fails && fail ('PG_TSMULTIRANGE returned wrong value: expected 4533, got ' . PG_TSMULTIRANGE));
PG_TSMULTIRANGEARRAY              ==  6152 or (++$fails && fail ('PG_TSMULTIRANGEARRAY returned wrong value: expected 6152, got ' . PG_TSMULTIRANGEARRAY));
PG_TSM_HANDLER                    ==  3310 or (++$fails && fail ('PG_TSM_HANDLER returned wrong value: expected 3310, got ' . PG_TSM_HANDLER));
PG_TSQUERY                        ==  3615 or (++$fails && fail ('PG_TSQUERY returned wrong value: expected 3615, got ' . PG_TSQUERY));
PG_TSQUERYARRAY                   ==  3645 or (++$fails && fail ('PG_TSQUERYARRAY returned wrong value: expected 3645, got ' . PG_TSQUERYARRAY));
PG_TSRANGE                        ==  3908 or (++$fails && fail ('PG_TSRANGE returned wrong value: expected 3908, got ' . PG_TSRANGE));
PG_TSRANGEARRAY                   ==  3909 or (++$fails && fail ('PG_TSRANGEARRAY returned wrong value: expected 3909, got ' . PG_TSRANGEARRAY));
PG_TSTZMULTIRANGE                 ==  4534 or (++$fails && fail ('PG_TSTZMULTIRANGE returned wrong value: expected 4534, got ' . PG_TSTZMULTIRANGE));
PG_TSTZMULTIRANGEARRAY            ==  6153 or (++$fails && fail ('PG_TSTZMULTIRANGEARRAY returned wrong value: expected 6153, got ' . PG_TSTZMULTIRANGEARRAY));
PG_TSTZRANGE                      ==  3910 or (++$fails && fail ('PG_TSTZRANGE returned wrong value: expected 3910, got ' . PG_TSTZRANGE));
PG_TSTZRANGEARRAY                 ==  3911 or (++$fails && fail ('PG_TSTZRANGEARRAY returned wrong value: expected 3911, got ' . PG_TSTZRANGEARRAY));
PG_TSVECTOR                       ==  3614 or (++$fails && fail ('PG_TSVECTOR returned wrong value: expected 3614, got ' . PG_TSVECTOR));
PG_TSVECTORARRAY                  ==  3643 or (++$fails && fail ('PG_TSVECTORARRAY returned wrong value: expected 3643, got ' . PG_TSVECTORARRAY));
PG_TXID_SNAPSHOT                  ==  2970 or (++$fails && fail ('PG_TXID_SNAPSHOT returned wrong value: expected 2970, got ' . PG_TXID_SNAPSHOT));
PG_TXID_SNAPSHOTARRAY             ==  2949 or (++$fails && fail ('PG_TXID_SNAPSHOTARRAY returned wrong value: expected 2949, got ' . PG_TXID_SNAPSHOTARRAY));
PG_UNKNOWN                        ==   705 or (++$fails && fail ('PG_UNKNOWN returned wrong value: expected 705, got ' . PG_UNKNOWN));
PG_UUID                           ==  2950 or (++$fails && fail ('PG_UUID returned wrong value: expected 2950, got ' . PG_UUID));
PG_UUIDARRAY                      ==  2951 or (++$fails && fail ('PG_UUIDARRAY returned wrong value: expected 2951, got ' . PG_UUIDARRAY));
PG_VARBIT                         ==  1562 or (++$fails && fail ('PG_VARBIT returned wrong value: expected 1562, got ' . PG_VARBIT));
PG_VARBITARRAY                    ==  1563 or (++$fails && fail ('PG_VARBITARRAY returned wrong value: expected 1563, got ' . PG_VARBITARRAY));
PG_VARCHAR                        ==  1043 or (++$fails && fail ('PG_VARCHAR returned wrong value: expected 1043, got ' . PG_VARCHAR));
PG_VARCHARARRAY                   ==  1015 or (++$fails && fail ('PG_VARCHARARRAY returned wrong value: expected 1015, got ' . PG_VARCHARARRAY));
PG_VOID                           ==  2278 or (++$fails && fail ('PG_VOID returned wrong value: expected 2278, got ' . PG_VOID));
PG_XID                            ==    28 or (++$fails && fail ('PG_XID returned wrong value: expected 28, got ' . PG_XID));
PG_XID8                           ==  5069 or (++$fails && fail ('PG_XID8 returned wrong value: expected 5069, got ' . PG_XID8));
PG_XID8ARRAY                      ==   271 or (++$fails && fail ('PG_XID8ARRAY returned wrong value: expected 271, got ' . PG_XID8ARRAY));
PG_XIDARRAY                       ==  1011 or (++$fails && fail ('PG_XIDARRAY returned wrong value: expected 1011, got ' . PG_XIDARRAY));
PG_XML                            ==   142 or (++$fails && fail ('PG_XML returned wrong value: expected 142, got ' . PG_XML));
PG_XMLARRAY                       ==   143 or (++$fails && fail ('PG_XMLARRAY returned wrong value: expected 143, got ' . PG_XMLARRAY));

if (!$fails) {
    pass ('All types returned the correct value');
}
